@@ -3,6 +3,7 @@ package catalog
 import (
 	"context"
 	"fmt"
+	"reflect"
 	"testing"
 	"time"
 
@@ -12,7 +13,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	utilclock "k8s.io/apimachinery/pkg/util/clock"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/reconciler"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry/resolver"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/fakes"
@@ -42,7 +43,7 @@ func TestSyncSubscriptions(t *testing.T) {
 		fields            fields
 		args              args
 		wantErr           error
-		wantInstallPlans  []v1alpha1.InstallPlan
+		wantInstallPlan   *v1alpha1.InstallPlan
 		wantSubscriptions []*v1alpha1.Subscription
 	}{
 		{
@@ -158,33 +159,31 @@ func TestSyncSubscriptions(t *testing.T) {
 					},
 				},
 			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{
-							"csv.v.1",
-						},
-						Approval:   v1alpha1.ApprovalAutomatic,
-						Approved:   true,
-						Generation: 1,
+			wantInstallPlan: &v1alpha1.InstallPlan{
+				Spec: v1alpha1.InstallPlanSpec{
+					ClusterServiceVersionNames: []string{
+						"csv.v.1",
 					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase: v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{
-							"src",
-						},
-						Plan: []*v1alpha1.Step{
-							{
-								Resolving: "csv.v.1",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "csv.v.1",
-									Manifest:               "{}",
-								},
+					Approval:   v1alpha1.ApprovalAutomatic,
+					Approved:   true,
+					Generation: 1,
+				},
+				Status: v1alpha1.InstallPlanStatus{
+					Phase: v1alpha1.InstallPlanPhaseInstalling,
+					CatalogSources: []string{
+						"src",
+					},
+					Plan: []*v1alpha1.Step{
+						{
+							Resolving: "csv.v.1",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.ClusterServiceVersionKind,
+								Name:                   "csv.v.1",
+								Manifest:               "{}",
 							},
 						},
 					},
@@ -301,33 +300,31 @@ func TestSyncSubscriptions(t *testing.T) {
 					},
 				},
 			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{
-							"csv.v.1",
-						},
-						Approval:   v1alpha1.ApprovalAutomatic,
-						Approved:   true,
-						Generation: 1,
+			wantInstallPlan: &v1alpha1.InstallPlan{
+				Spec: v1alpha1.InstallPlanSpec{
+					ClusterServiceVersionNames: []string{
+						"csv.v.1",
 					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase: v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{
-							"src",
-						},
-						Plan: []*v1alpha1.Step{
-							{
-								Resolving: "csv.v.1",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "csv.v.1",
-									Manifest:               "{}",
-								},
+					Approval:   v1alpha1.ApprovalAutomatic,
+					Approved:   true,
+					Generation: 1,
+				},
+				Status: v1alpha1.InstallPlanStatus{
+					Phase: v1alpha1.InstallPlanPhaseInstalling,
+					CatalogSources: []string{
+						"src",
+					},
+					Plan: []*v1alpha1.Step{
+						{
+							Resolving: "csv.v.1",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.ClusterServiceVersionKind,
+								Name:                   "csv.v.1",
+								Manifest:               "{}",
 							},
 						},
 					},
@@ -380,7 +377,7 @@ func TestSyncSubscriptions(t *testing.T) {
 				},
 				bundleLookups: []v1alpha1.BundleLookup{
 					{
-						Path:       "bundle-path-a",
+						Path: "bundle-path-a",
 						Identifier: "bundle-a",
 						CatalogSourceRef: &corev1.ObjectReference{
 							Namespace: testNamespace,
@@ -449,33 +446,31 @@ func TestSyncSubscriptions(t *testing.T) {
 					},
 				},
 			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{"bundle-a"},
-						Approval:                   v1alpha1.ApprovalAutomatic,
-						Approved:                   true,
-						Generation:                 1,
-					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase:          v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{},
-						BundleLookups: []v1alpha1.BundleLookup{
-							{
-								Path:       "bundle-path-a",
-								Identifier: "bundle-a",
-								CatalogSourceRef: &corev1.ObjectReference{
-									Namespace: testNamespace,
-									Name:      "src",
-								},
-								Conditions: []v1alpha1.BundleLookupCondition{
-									{
-										Type:               v1alpha1.BundleLookupPending,
-										Status:             corev1.ConditionTrue,
-										Reason:             "JobIncomplete",
-										Message:            "unpack job not completed",
-										LastTransitionTime: &now,
-									},
+			wantInstallPlan: &v1alpha1.InstallPlan{
+				Spec: v1alpha1.InstallPlanSpec{
+					ClusterServiceVersionNames: []string{"bundle-a"},
+					Approval:                   v1alpha1.ApprovalAutomatic,
+					Approved:                   true,
+					Generation:                 1,
+				},
+				Status: v1alpha1.InstallPlanStatus{
+					Phase:          v1alpha1.InstallPlanPhaseInstalling,
+					CatalogSources: []string{},
+					BundleLookups: []v1alpha1.BundleLookup{
+						{
+							Path: "bundle-path-a",
+							Identifier: "bundle-a",
+							CatalogSourceRef: &corev1.ObjectReference{
+								Namespace: testNamespace,
+								Name:      "src",
+							},
+							Conditions: []v1alpha1.BundleLookupCondition{
+								{
+									Type:               v1alpha1.BundleLookupPending,
+									Status:             corev1.ConditionTrue,
+									Reason:             "JobIncomplete",
+									Message:            "unpack job not completed",
+									LastTransitionTime: &now,
 								},
 							},
 						},
@@ -602,33 +597,31 @@ func TestSyncSubscriptions(t *testing.T) {
 					},
 				},
 			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{
-							"csv.v.2",
-						},
-						Approval:   v1alpha1.ApprovalAutomatic,
-						Approved:   true,
-						Generation: 1,
+			wantInstallPlan: &v1alpha1.InstallPlan{
+				Spec: v1alpha1.InstallPlanSpec{
+					ClusterServiceVersionNames: []string{
+						"csv.v.2",
 					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase: v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{
-							"src",
-						},
-						Plan: []*v1alpha1.Step{
-							{
-								Resolving: "csv.v.2",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "csv.v.2",
-									Manifest:               "{}",
-								},
+					Approval:   v1alpha1.ApprovalAutomatic,
+					Approved:   true,
+					Generation: 1,
+				},
+				Status: v1alpha1.InstallPlanStatus{
+					Phase: v1alpha1.InstallPlanPhaseInstalling,
+					CatalogSources: []string{
+						"src",
+					},
+					Plan: []*v1alpha1.Step{
+						{
+							Resolving: "csv.v.2",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.ClusterServiceVersionKind,
+								Name:                   "csv.v.2",
+								Manifest:               "{}",
 							},
 						},
 					},
@@ -778,237 +771,56 @@ func TestSyncSubscriptions(t *testing.T) {
 					},
 				},
 			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{
-							"csv.v.2",
-							"dep.v.1",
-						},
-						Approval:   v1alpha1.ApprovalAutomatic,
-						Approved:   true,
-						Generation: 1,
+			wantInstallPlan: &v1alpha1.InstallPlan{
+				Spec: v1alpha1.InstallPlanSpec{
+					ClusterServiceVersionNames: []string{
+						"csv.v.2",
+						"dep.v.1",
 					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase: v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{
-							"src",
-						},
-						Plan: []*v1alpha1.Step{
-							{
-								Resolving: "csv.v.2",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "csv.v.2",
-									Manifest:               "{}",
-								},
-							},
-							{
-								Resolving: "csv.v.2",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "dep.v.1",
-									Manifest:               "{}",
-								},
-							},
-							{
-								Resolving: "csv.v.2",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.SubscriptionKind,
-									Name:                   "sub-dep",
-									Manifest:               "{}",
-								},
+					Approval:   v1alpha1.ApprovalAutomatic,
+					Approved:   true,
+					Generation: 1,
+				},
+				Status: v1alpha1.InstallPlanStatus{
+					Phase: v1alpha1.InstallPlanPhaseInstalling,
+					CatalogSources: []string{
+						"src",
+					},
+					Plan: []*v1alpha1.Step{
+						{
+							Resolving: "csv.v.2",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.ClusterServiceVersionKind,
+								Name:                   "csv.v.2",
+								Manifest:               "{}",
 							},
 						},
-					},
-				},
-			},
-		},
-		{
-			name: "ExistingInstallPlanGenerationRespected",
-			fields: fields{
-				clientOptions: []clientfake.Option{clientfake.WithSelfLinks(t), clientfake.WithNameGenerationList(t, "install-gen-2")},
-				existingOLMObjs: []runtime.Object{
-					&v1alpha1.Subscription{
-						TypeMeta: metav1.TypeMeta{
-							Kind:       v1alpha1.SubscriptionKind,
-							APIVersion: v1alpha1.SchemeGroupVersion.String(),
-						},
-						ObjectMeta: metav1.ObjectMeta{
-							Name:      "sub",
-							Namespace: testNamespace,
-						},
-						Spec: &v1alpha1.SubscriptionSpec{
-							CatalogSource:          "src",
-							CatalogSourceNamespace: testNamespace,
-						},
-						Status: v1alpha1.SubscriptionStatus{
-							CurrentCSV: "",
-							State:      "",
-						},
-					},
-					&v1alpha1.InstallPlan{
-						ObjectMeta: metav1.ObjectMeta{
-							Name:      "ip",
-							Namespace: testNamespace,
-						},
-						Spec: v1alpha1.InstallPlanSpec{
-							Approval: v1alpha1.ApprovalAutomatic,
-							Approved: true,
-							ClusterServiceVersionNames: []string{
-								"some-csv",
+						{
+							Resolving: "csv.v.2",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.ClusterServiceVersionKind,
+								Name:                   "dep.v.1",
+								Manifest:               "{}",
 							},
-							// Claim the gen 1 to ensure a new InstallPlan is created at gen 2
-							Generation: 1,
 						},
-					},
-				},
-				resolveSteps: []*v1alpha1.Step{
-					{
-						Resolving: "csv.v.1",
-						Resource: v1alpha1.StepResource{
-							CatalogSource:          "src",
-							CatalogSourceNamespace: testNamespace,
-							Group:                  v1alpha1.GroupName,
-							Version:                v1alpha1.GroupVersion,
-							Kind:                   v1alpha1.ClusterServiceVersionKind,
-							Name:                   "csv.v.1",
-							Manifest:               "{}",
-						},
-					},
-				},
-				resolveSubs: []*v1alpha1.Subscription{
-					{
-						TypeMeta: metav1.TypeMeta{
-							Kind:       v1alpha1.SubscriptionKind,
-							APIVersion: v1alpha1.SchemeGroupVersion.String(),
-						},
-						ObjectMeta: metav1.ObjectMeta{
-							Name:      "sub",
-							Namespace: testNamespace,
-						},
-						Spec: &v1alpha1.SubscriptionSpec{
-							CatalogSource:          "src",
-							CatalogSourceNamespace: testNamespace,
-						},
-						Status: v1alpha1.SubscriptionStatus{
-							CurrentCSV: "csv.v.1",
-							State:      "SubscriptionStateAtLatest",
-						},
-					},
-				},
-			},
-			args: args{
-				obj: &v1alpha1.Subscription{
-					TypeMeta: metav1.TypeMeta{
-						Kind:       v1alpha1.SubscriptionKind,
-						APIVersion: v1alpha1.SchemeGroupVersion.String(),
-					},
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "sub",
-						Namespace: testNamespace,
-					},
-					Spec: &v1alpha1.SubscriptionSpec{
-						CatalogSource:          "src",
-						CatalogSourceNamespace: testNamespace,
-					},
-					Status: v1alpha1.SubscriptionStatus{
-						CurrentCSV: "",
-						State:      "",
-					},
-				},
-			},
-			wantSubscriptions: []*v1alpha1.Subscription{
-				{
-					TypeMeta: metav1.TypeMeta{
-						Kind:       v1alpha1.SubscriptionKind,
-						APIVersion: v1alpha1.SubscriptionCRDAPIVersion,
-					},
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "sub",
-						Namespace: testNamespace,
-					},
-					Spec: &v1alpha1.SubscriptionSpec{
-						CatalogSource:          "src",
-						CatalogSourceNamespace: testNamespace,
-					},
-					Status: v1alpha1.SubscriptionStatus{
-						CurrentCSV: "csv.v.1",
-						State:      v1alpha1.SubscriptionStateUpgradePending,
-						Install: &v1alpha1.InstallPlanReference{
-							Name:       "install-gen-2",
-							Kind:       v1alpha1.InstallPlanKind,
-							APIVersion: v1alpha1.InstallPlanAPIVersion,
-						},
-						InstallPlanRef: &corev1.ObjectReference{
-							Name:       "install-gen-2",
-							Namespace:  testNamespace,
-							Kind:       v1alpha1.InstallPlanKind,
-							APIVersion: v1alpha1.InstallPlanAPIVersion,
-						},
-						LastUpdated:           now,
-						InstallPlanGeneration: 2,
-					},
-				},
-			},
-			wantInstallPlans: []v1alpha1.InstallPlan{
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "ip",
-						Namespace: testNamespace,
-					},
-					Spec: v1alpha1.InstallPlanSpec{
-						Approval: v1alpha1.ApprovalAutomatic,
-						Approved: true,
-						ClusterServiceVersionNames: []string{
-							"some-csv",
-						},
-						// Claim the gen 1 to ensure a new InstallPlan is created at gen 2
-						Generation: 1,
-					},
-				},
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "install-gen-2",
-						Namespace: testNamespace,
-					},
-					Spec: v1alpha1.InstallPlanSpec{
-						ClusterServiceVersionNames: []string{
-							"csv.v.1",
-						},
-						Approval:   v1alpha1.ApprovalAutomatic,
-						Approved:   true,
-						Generation: 2,
-					},
-					Status: v1alpha1.InstallPlanStatus{
-						Phase: v1alpha1.InstallPlanPhaseInstalling,
-						CatalogSources: []string{
-							"src",
-						},
-						Plan: []*v1alpha1.Step{
-							{
-								Resolving: "csv.v.1",
-								Resource: v1alpha1.StepResource{
-									CatalogSource:          "src",
-									CatalogSourceNamespace: testNamespace,
-									Group:                  v1alpha1.GroupName,
-									Version:                v1alpha1.GroupVersion,
-									Kind:                   v1alpha1.ClusterServiceVersionKind,
-									Name:                   "csv.v.1",
-									Manifest:               "{}",
-								},
+						{
+							Resolving: "csv.v.2",
+							Resource: v1alpha1.StepResource{
+								CatalogSource:          "src",
+								CatalogSourceNamespace: testNamespace,
+								Group:                  v1alpha1.GroupName,
+								Version:                v1alpha1.GroupVersion,
+								Kind:                   v1alpha1.SubscriptionKind,
+								Name:                   "sub-dep",
+								Manifest:               "{}",
 							},
 						},
 					},
@@ -1054,24 +866,20 @@ func TestSyncSubscriptions(t *testing.T) {
 			}
 
 			for _, s := range tt.wantSubscriptions {
-				fetched, err := o.client.OperatorsV1alpha1().Subscriptions(testNamespace).Get(s.GetName(), metav1.GetOptions{})
+				fetched, err := o.client.OperatorsV1alpha1().Subscriptions(testNamespace).Get(context.TODO(), s.GetName(), metav1.GetOptions{})
 				require.NoError(t, err)
 				require.Equal(t, s, fetched)
 			}
-
-			installPlans, err := o.client.OperatorsV1alpha1().InstallPlans(testNamespace).List(metav1.ListOptions{})
-			require.Len(t, installPlans.Items, len(tt.wantInstallPlans))
-
-			haveIPs := make(map[string]v1alpha1.InstallPlan)
-			for _, ip := range installPlans.Items {
-				haveIPs[ip.GetName()] = ip
-			}
-
-			for _, ip := range tt.wantInstallPlans {
-				have, ok := haveIPs[ip.GetName()]
-				require.True(t, ok, "installplan %s missing", ip.GetName())
-				require.Equal(t, have.Spec, ip.Spec)
-				require.Equal(t, have.Status, ip.Status)
+			if tt.wantInstallPlan != nil {
+				installPlans, err := o.client.OperatorsV1alpha1().InstallPlans(testNamespace).List(context.TODO(), metav1.ListOptions{})
+				require.NoError(t, err)
+				require.Equal(t, 1, len(installPlans.Items))
+				ip := installPlans.Items[0]
+				if (!reflect.DeepEqual(tt.wantInstallPlan.ObjectMeta, metav1.ObjectMeta{})) {
+					require.Equal(t, tt.wantInstallPlan.ObjectMeta, ip.ObjectMeta)
+				}
+				require.Equal(t, tt.wantInstallPlan.Spec, ip.Spec)
+				require.Equal(t, tt.wantInstallPlan.Status, ip.Status)
 			}
 		})
 	}
