@@ -3,8 +3,8 @@ package ownerutil
 import (
 	"fmt"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1"
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	"github.com/operator-framework/api/pkg/operators/v1"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 
 	log "github.com/sirupsen/logrus"
 	corev1 "k8s.io/api/core/v1"
@@ -314,6 +314,18 @@ func InferGroupVersionKind(obj runtime.Object) error {
 			Version: "v1",
 			Kind:    "ServiceAccount",
 		})
+	case *corev1.ConfigMap:
+		objectKind.SetGroupVersionKind(schema.GroupVersionKind{
+			Group:   "",
+			Version: "v1",
+			Kind:    "ConfigMap",
+		})
+	case *corev1.Secret:
+		objectKind.SetGroupVersionKind(schema.GroupVersionKind{
+			Group:   "",
+			Version: "v1",
+			Kind:    "Secret",
+		})
 	case *rbac.ClusterRole:
 		objectKind.SetGroupVersionKind(schema.GroupVersionKind{
 			Group:   "rbac.authorization.k8s.io",
